# frozen_string_literal: true

# Handle eligible positions
class UpdateEligibleJob < ActiveJob::Base
  queue_as :rlow

  def perform(score_id)
    # kill other pending jobs
    Resque::Job.destroy('rlow', UpdateEligibleJob, score_id)
    if score = Hermes::Score.find_by(id: score_id)
      last_pos=score.last_eligible_position
      base_query = {
        housing_offer_id: score.housing_offer_id
      }

      sne_date_to_compare = score.sne_date

      query, increment = if last_pos.nil? || last_pos > score.eligible_position
                           if (same_pos_score = Hermes::Score.find_by(
                             id: { :$ne => score.id },
                             housing_offer_id: score.housing_offer_id,
                             eligible_position: score.eligible_position,
                             profile_epci_score: score.profile_epci_score
                           ))
                             sne_date_to_compare = [same_pos_score.sne_date, score.sne_date].min
                           end
                           inter = base_query.merge(
                             :$or => [
                               {
                                 eligible_position: {
                                   :$gt => score.eligible_position
                                 }
                               },
                               {
                                 eligible_position: score.eligible_position,
                                 profile_epci_score: { :$lt => score.profile_epci_score }
                               },
                               {
                                 eligible_position: score.eligible_position,
                                 profile_epci_score: score.profile_epci_score,
                                 sne_date: { :$gt => sne_date_to_compare }
                               }
                             ]
                           )
                           inter[:eligible_position] = { :$lt => last_pos } if last_pos
                           [inter, 1]
                         elsif last_pos < score.eligible_position
                           if (same_pos_score = Hermes::Score.find_by(
                             id: { :$ne => score.id },
                             housing_offer_id: score.housing_offer_id,
                             eligible_position: score.eligible_position,
                             profile_epci_score: score.profile_epci_score
                           ))
                             sne_date_to_compare = [same_pos_score.sne_date, score.sne_date].max
                           end
                           [
                             base_query.merge(
                               eligible_position: {
                                 :$gt => last_pos
                               },
                               :$or => [
                                 {
                                   eligible_position: {
                                     :$lt => score.eligible_position
                                   }
                                 },
                                 {
                                   eligible_position: score.eligible_position,
                                   profile_epci_score: { :$gt => score.profile_epci_score }
                                 },
                                 {
                                   eligible_position: score.eligible_position,
                                   profile_epci_score: score.profile_epci_score,
                                   sne_date: { :$lt => sne_date_to_compare }
                                 }
                               ]
                             ),
                             -1
                           ]
                         end
      scores = Hermes::Score.where(query)
      scores.inc(eligible_position: increment)
      scores.set(updated_at: DateTime.current)
    end
  end
end
